# Copyright (C) 2021  https://seetee.io

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import base64
import codecs
import json
import os

import grpc

from .generated import lightning_pb2 as ln
from .generated import lightning_pb2_grpc as lnrpc
from .tlv_keys import TLVKeys


def load_custom_records(
    tls_cert_path, macaroon_path, rpc_host, tlv_keys, index_offset, max_invoices
):
    os.environ["GRPC_SSL_CIPHER_SUITES"] = 'HIGH+ECDSA'

    with open(tls_cert_path, 'rb') as f:
        cert = f.read()
        creds = grpc.ssl_channel_credentials(cert)

    with open(macaroon_path, 'rb') as f:
        macaroon_bytes = f.read()
        macaroon = codecs.encode(macaroon_bytes, 'hex')

    with grpc.secure_channel(rpc_host, creds) as channel:
        stub = lnrpc.LightningStub(channel)

        request = ln.ListInvoiceRequest(
            pending_only=False,
            index_offset=index_offset,
            num_max_invoices=max_invoices,
            reversed=False
        )

        response = stub.ListInvoices(request, metadata=[('macaroon', macaroon)])
        invoices = response.invoices

        first_index = lastindex = None
        if len(invoices) > 0:
            first_index = invoices[0].add_index
            lastindex = invoices[-1].add_index

        custom_records = [
            _decode_tlv(key, value)
            for invoice in invoices
            for htlc in invoice.htlcs if invoice.is_keysend
            for key, value in htlc.custom_records.items() if key in tlv_keys
        ]

        return {
            'custom_records': custom_records,
            'queried_invoices': {
                'first_index': first_index,
                'last_index': lastindex
            }
        }

def _decode_tlv(key, value):
    decoders = {
        TLVKeys.PODCAST_METADATA.value:
        lambda v: json.loads(v.decode(encoding="UTF-8")),
    }

    try:
        return {'key': key, 'value': decoders[key](value)}
    except KeyError:
        # default to base64 for unknown TLV keys
        return {'key': key, 'value': base64.b64encode(value).decode("UTF-8")}



import os

import setuptools


def get_long_description():
    with open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md"),
        encoding="utf8",
    ) as fp:
        return fp.read()

setuptools.setup(
    name='lntlvs',
    version='0.0.1',
    description=
    'Extract custom metadata records that may have been attached to lightning payments you received',
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    url='https://github.com/seetee-io/lntlvs',
    author='@dnlggr',
    author_email='hello@seetee.io',
    license='GPL-3.0-only',
    entry_points={
        'console_scripts': ['lntlvs=lntlvs.cli:main'],
    },
    install_requires=[
        'click>=8.0.3,<9', 'grpcio>=1.42,<2',
        'googleapis-common-protos>=1.53,<2'
    ],
    extras_require={'dev': ['grpcio-tools>=1.42,<2']},
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.7",
)



# Copyright (C) 2021  https://seetee.io

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import json
import os
import sys

import click

from . import load_custom_records
from .tlv_keys import TLVKeys


@click.command(
    short_help=
    "Extract custom metadata records that may have been attached to lightning payments you received."
)
@click.option(
    '--tlscertpath',
    default='~/.lnd/tls.cert',
    show_default=True,
    metavar='<value>',
    help="\b\nPath to the TLS certificate for lnd's RPC service"
)
@click.option(
    '--macaroonpath',
    default='~/.lnd/data/chain/bitcoin/mainnet/readonly.macaroon',
    show_default=True,
    metavar='<value>',
    help="\b\nPath to the (read-only) macaroon for lnd's RPC service"
)
@click.option(
    '--rpchost',
    default='127.0.0.1:10009',
    show_default=True,
    metavar='<value>',
    help="\b\nThe address and port where lnd's RPC service is listening"
)
@click.option(
    '--tlvkey',
    multiple=True,
    default=[TLVKeys.PODCAST_METADATA.value],
    show_default=True,
    metavar='<value>',
    help="\b\nA TLV key to extract - can be specified multiple times"
)
@click.option(
    '--indexoffset',
    default=0,
    show_default=True,
    metavar='<value>',
    help="\b\nQuery for invoices starting at (not including) this offset"
)
@click.option(
    '--maxinvoices',
    default=1000,
    show_default=True,
    metavar='<value>',
    help="\b\nThe maximum number of invoices to query"
)
def main(tlscertpath, macaroonpath, rpchost, tlvkey, indexoffset, maxinvoices):
    """\b
    Extract custom metadata records that may have been attached to lightning payments you received.
    """

    tlvkeys = [*tlvkey]

    custom_records = load_custom_records(
        tls_cert_path=os.path.expanduser(tlscertpath),
        macaroon_path=os.path.expanduser(macaroonpath),
        rpc_host=rpchost,
        tlv_keys=tlvkeys,
        index_offset=indexoffset,
        max_invoices=maxinvoices
    )

    click.echo(json.dumps(custom_records, indent=4))
